library engenius;

import 'dart:async';
import 'dart:typed_data';
import 'dart:convert';
import 'servicioPost.dart';
import 'package:flutter/services.dart';

/// Esta clase permite la comunicación con el lector de tarjetas de proximidad
/// de la marca Engenius
///
/// Para poder utilizarla es necesario importar el paquete en el archivo
/// de inicio de la aplicación.
///
/// ```dart
/// import 'package:engenius/engenius.dart';
/// ```
///
/// Para poder utilizarla es necesario instanciar la clase y llamar el metodo
/// que se necesite.
///
/// ```dart
/// Engenius engenius = Engenius();
/// var resultado = await engenius.isMF1Here;
/// ```
///
/// Los metodos disponibles son los siguientes:
///
/// * [isMF1Here] - Devuelve un booleano indicando si hay una tarjeta
/// de proximidad cerca del lector.
/// * [habilitar] - Habilita el lector para la lectura de tarjetas.
/// * [leerEnviar] - Devuelve un [List] con la información de la tarjeta
/// leida o un [List] vacio si no se pudo leer la tarjeta.
/// * [recargar] - Devuelve un [String] con la respuesta del servidor
/// de recarga, si la recarga es exitosa devuelve la palabra "true"
/// de lo contrario devuelve "false".

class Engenius {
  static const MethodChannel channel =
      const MethodChannel('flutter_nfc_mifare');
  static String idcard = '';
  static Future<bool?> get isMF1Here async {
    return await channel.invokeMethod('isMF1Here');
  }

  static Future<bool?> get habilitar async {
    return await channel.invokeMethod('habilitar');
  }

  static Future<List> leerEnviar() async {
    //Uint8List key = Uint8List.fromList([255, 255, 255, 255, 255, 255]);
    Uint8List key = Uint8List.fromList([255, 255, 255, 255, 255, 1]);
    print('Esta es la llave de acceso: ${key.toString()}');
    var resultado = await readMF1(1, 1, key);
    var data = resultado
        .join()
        .toString()
        .replaceAll("[", "")
        .replaceAll("]", "")
        .replaceAll(" ", "");
    if (data == "1") {
      print("TARJETA NUEVA &/&/&/&/&/&/&/&/&/&/&/&/&/&/&/&/&/&/");
      return ["NEW"];
    } else if (data == "2") {
      print("TARJETA NO ASIGNADAAAAAAAA &/&/&/&/&/&/&/&/&/&/&/&/&/&/&/&/&/&/");
      return ["NOA"];
    } else if (data == "0") {
      print("NO LECTURA &/&/&/&/&/&/&/&/&/&/&/&/&/&/&/&/&/&/");
      return ["NLE"];
    } else {
      var _idcard = data.substring(0, 8);
      idcard = _idcard;
      var jsonObj = jsonEncode(<String, String>{
        "option": "infoinicial",
        "IdCard": _idcard,
        "DataCard": data
      });
      print(jsonDecode(jsonObj));
      var urlNew = Uri.https(
          'www.php.engenius.com.co', '/Recargas/DatabaseRecargas4k.php');
      PostData dataCoin = PostData(urlAPI: urlNew, bodyObj: jsonObj);
      var dataLocal = await dataCoin.getCoinData();
      print('dataLocal: $dataLocal');
      if (dataLocal['auten'][0][0] == 'VRD') {
        print('Cambiar de pagina');
        if (dataLocal['Beneficiario'][0] == null) {
          print("USUARIO NO REGISTRADO EN LA PLATAFORMA");
          var respuestamal = ["NOR"];
          return respuestamal;
        } else {
          var respuestaok = [
            dataLocal['Beneficiario'][0][1].toString(),
            dataLocal['Beneficiario'][0][2].toString(),
            dataLocal['Beneficiario'][0][4].toString(),
            dataLocal['Beneficiario'][0][5].toString(),
            dataLocal['plans']['cost'][0][0].toString(),
            _idcard
          ];
          print("Usuario: " +
              dataLocal['Beneficiario'][0][1].toString() +
              '\r\n' +
              "Documento: " +
              dataLocal['Beneficiario'][0][2].toString() +
              '\r\n' +
              "Tipo Instalación: " +
              dataLocal['Beneficiario'][0][4].toString() +
              '\r\n' +
              "Ultima Recarga: " +
              dataLocal['Beneficiario'][0][5].toString() +
              '\r\n' +
              "Valor a Pagar: " +
              dataLocal['plans']['cost'][0].toString() +
              '\r\n');
          return respuestaok;
        }
      }
    }
    return [];
  }

  static Future<String> recargar(idcard, valor, pass, usuario) async {
    try {
      var jsonObj = jsonEncode(<String, String>{
        "option": "recarga",
        "IdCard": idcard,
        "Password": pass,
        "Topup_Quantity": "30",
        "Topup_Type_Id": "1",
        "Topup_Value": valor.toString(),
        "User": usuario
      });
      print(jsonDecode(jsonObj));
      var urlNew = Uri.https(
          "www.php.engenius.com.co", '/Recargas/DatabaseRecargas4k.php');

      PostData dataCoin = PostData(urlAPI: urlNew, bodyObj: jsonObj);
      var dataLocal = await dataCoin.getCoinData();
      print('dataLocal: $dataLocal');
      if (dataLocal['auten'][0][0] == 'VRD') {
        print('TOKEN');
        var respuesta = await write(dataLocal['Token'][0].toString());
        if (respuesta == "true") {
          return "true";
        } else {
          return "false";
        }
        //"PIN DE RECARGA: " + dataLocal['Token'][0].toString() + '\r\n';
        //_token = dataLocal['Token'][0].toString();
      } else {
        return "false";
      }
    } catch (e) {
      print('Error: $e');
      return "false";
    }
  }

  static Future<List> readMF1(
      int sectorIndex, int blockIndex, Uint8List key) async {
    var llaves = await channel.invokeMethod('readMF1', [3, 0, key]);
    if (llaves == null) {
      key = Uint8List.fromList([255, 255, 255, 255, 255, 255]);
      print('Tarjeta Nueva');
      return [1];
    } else {
      var tipo = await channel.invokeMethod('readMF1', [39, 0, key]);
      print("TIPO TARJETA");
      print(tipo);
      if (tipo == null) {
        var _resultado = new List.filled(
            64, [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]);
        var _lectura;
        var cont = 0;
        for (var i = 0; i < 16; i++) {
          for (var j = 0; j < 4; j++) {
            /*if (i == 1 || i == 0) {
            _lectura = await channel.invokeMethod('readMF1', [
              i,
              j,
              Uint8List.fromList([255, 255, 255, 255, 255, 255])
            ]);
          } else {
            _lectura = await channel.invokeMethod('readMF1', [i, j, key]);
          }*/
            _lectura = await channel.invokeMethod('readMF1', [i, j, key]);
            print('Sector' +
                i.toString() +
                'Bloque' +
                j.toString() +
                '-' +
                _lectura.toString());
            if (_lectura == null) {
              _lectura = 'Login Fail';
            }
            try {
              _resultado[cont] = _lectura;
              cont = cont + 1;
            } catch (e) {
              print(e.toString() + '-' + i.toString() + '-' + j.toString());
              return [0];
            }
            if (cont == 64) {
              print('Lectura Completa');
              return await procesar(_resultado);
            }
          }
        }
      } else {
        var serial = await channel.invokeMethod('readMF1', [9, 1, key]);
        print(serial);
        if (serial[3] != 5) {
          print("Tarjeta No Asignada");
          return [2];
        } else if (serial == null) {
          print('Tarjeta Nueva');
          return [1];
        } else {
          var _resultado = new List.filled(
              256, [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]);
          var _lectura;
          var cont = 0;
          for (var i = 0; i < 32; i++) {
            for (var j = 0; j < 4; j++) {
              if (i == 1 || i == 0) {
                _lectura = await channel.invokeMethod('readMF1', [
                  i,
                  j,
                  Uint8List.fromList([255, 255, 255, 255, 255, 255])
                ]);
              } else {
                _lectura = await channel.invokeMethod('readMF1', [i, j, key]);
              }
              print(_lectura);
              if (_lectura == null) {
                _lectura = 'Login Fail';
              }
              try {
                _resultado[cont] = _lectura;
                cont = cont + 1;
              } catch (e) {
                print(e.toString() + '-' + i.toString() + '-' + j.toString());
                return [0];
              }
              if (cont == 128) {
                for (var k = 32; k < 40; k++) {
                  for (var h = 0; h < 16; h++) {
                    _lectura =
                        await channel.invokeMethod('readMF1', [k, h, key]);
                    print(_lectura);
                    if (_lectura == null) {
                      _lectura = 'Login Fail';
                    }
                    try {
                      _resultado[cont] = _lectura;
                      cont = cont + 1;
                    } catch (e) {
                      print(e.toString() +
                          '-' +
                          i.toString() +
                          '-' +
                          j.toString());
                      return [0];
                    }
                    if (cont == 256) {
                      print('Lectura Terminada');
                      return await procesar4k(_resultado);
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
    return [];
  }

  static Future<List> procesar(List resultado) async {
    var listado = new List.filled(64, "");
    var cont = 0;
    var texto = '';
    for (var i = 0; i < resultado.length; i++) {
      texto = '';
      if (resultado[i] == 'Login Fail') {
        texto = 'Login Fail';
      } else {
        for (var j = 0; j < 16; j++) {
          var prov = resultado[i][j].toRadixString(16);
          print(prov.length);
          if (prov.length == 1) {
            texto = texto + '0' + resultado[i][j].toRadixString(16);
          } else {
            texto = texto + resultado[i][j].toRadixString(16);
          }
        }
      }
      listado[i] = texto + '';
      cont = cont + 1;
      if (cont == 64) {
        return listado;
      }
    }
    return [];
  }

  static Future<List> procesar4k(List resultado) async {
    var listado = new List.filled(256, "");
    var cont = 0;
    var texto = '';
    for (var i = 0; i < resultado.length; i++) {
      texto = '';
      if (resultado[i] == 'Login Fail') {
        texto = 'Login Fail';
      } else {
        for (var j = 0; j < 16; j++) {
          var prov = resultado[i][j].toRadixString(16);
          print(prov.length);
          if (prov.length == 1) {
            texto = texto + '0' + resultado[i][j].toRadixString(16);
          } else {
            texto = texto + resultado[i][j].toRadixString(16);
          }
        }
      }
      listado[i] = texto + '';
      cont = cont + 1;
      if (cont == 256) {
        return listado;
      }
    }
    return [];
  }

  static Future<String> write(token) async {
    Uint8List key = Uint8List.fromList([255, 255, 255, 255, 255, 1]);
    print('Esta es la llave de acceso: ${key.toString()}');
    await writeMF1(token, key).then((value) {
      print("RESPUESTA ESCRIBIR");
      print(value);
      if (value == true) {
        return "true";
      } else {
        return "false";
      }
    });
    return "false";
  }

  static Future<bool> writeMF1(String token, Uint8List key) async {
    //var listado = new List.filled(5,[]);
    var listadoDef = new List.filled(5, []);
    var b1 = token.substring(0, 32);
    var b2 = token.substring(32, 64);
    var b3 = token.substring(64, 96);
    var b4 = token.substring(96, 128);
    var b5 = token.substring(128, 160);
    var listado = [b1, b2, b3, b4, b5];
    print(listado.toString());
    for (var i = 0; i < listado.length; i++) {
      listadoDef[i] = new Uint8List(16);
      for (var j = 0; j < 16; j++) {
        listadoDef[i][j] =
            int.parse(listado[i].substring(j * 2, (j * 2) + 2), radix: 16);
      }
    }
    print(listadoDef.toString());
    var result = await channel
        .invokeMethod('writeMF1', [0, 1, key, listadoDef[0]])
        .then((value) => channel
            .invokeMethod('writeMF1', [0, 2, key, listadoDef[1]])
            .then((value) => channel
                .invokeMethod('writeMF1', [1, 0, key, listadoDef[2]])
                .then((value) => channel
                    .invokeMethod('writeMF1', [1, 1, key, listadoDef[3]])
                    .then((value) => channel.invokeMethod('writeMF1',
                            [1, 2, key, listadoDef[4]]).then((value) {
                          print('TERMINE TODO');
                          return true;
                        }).catchError((error) => error))
                    .catchError((error) => error))
                .catchError((error) => error))
            .catchError((error) => error))
        .catchError((error) => error);

    return result;
  }

  static Future<bool?> writeMF2(
      int sectorIndex, int blockIndex, Uint8List key, Uint8List data) async {
    return await channel
        .invokeMethod('writeMF1', [sectorIndex, blockIndex, key, data]);
  }
}
